@@ -10,6 +10,8 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2011      Oracle and/or its affiliates.  All rights reserved. * $COPYRIGHT$
+ * Copyright (c)      2011 Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
  * 
  * Additional copyrights may follow
  * 
@@ -56,7 +58,6 @@
 #if ORTE_ENABLE_MULTICAST
 #include "orte/mca/rmcast/base/base.h"
 #endif
-#include "orte/mca/debugger/base/base.h"
 
 #include "orte/mca/rmaps/base/base.h"
 #if OPAL_ENABLE_FT_CR == 1
@@ -114,7 +115,6 @@ static int rte_init(void)
     orte_job_t *jdata;
     orte_node_t *node;
     orte_proc_t *proc;
-    int value;
     
     /* initialize the global list of local children and job data */
     OBJ_CONSTRUCT(&orte_local_children, opal_list_t);
@@ -126,27 +126,6 @@ static int rte_init(void)
         goto error;
     }
     
-    /* determine the topology info */
-    if (0 == orte_default_num_sockets_per_board) {
-        /* we weren't given a number, so try to determine it */
-        if (OPAL_SUCCESS != opal_paffinity_base_get_socket_info(&value)) {
-            /* can't get any info - default to 1 */
-            value = 1;
-        }
-        orte_default_num_sockets_per_board = (uint8_t)value;
-    }
-    if (0 == orte_default_num_cores_per_socket) {
-        /* we weren't given a number, so try to determine it */
-        if (OPAL_SUCCESS != (ret = opal_paffinity_base_get_core_info(0, &value))) {
-            /* don't have topo info - can we at least get #processors? */
-            if (OPAL_SUCCESS != opal_paffinity_base_get_processor_info(&value)) {
-                /* can't get any info - default to 1 */
-                value = 1;
-            }
-        }
-        orte_default_num_cores_per_socket = (uint8_t)value;
-    }
-    
     /* if we are using xml for output, put an mpirun start tag */
     if (orte_xml_output) {
         fprintf(orte_xml_fp, "<mpirun>\n");
@@ -526,18 +505,6 @@ static int rte_init(void)
         goto error;
     }
 
-    /* start the debuggers */ 
-    if (ORTE_SUCCESS != (ret = orte_debugger_base_open())) {
-      ORTE_ERROR_LOG(ret);
-      error = "orte_debugger_open";
-      goto error;
-    }
-    if (ORTE_SUCCESS != (ret = orte_debugger_base_select())) {
-      ORTE_ERROR_LOG(ret);
-      error = "orte_debugger_select";
-      goto error;
-    }
-
     /* if a tool has launched us and is requesting event reports,
      * then set its contact info into the comm system
      */
